       IDENTIFICATION DIVISION.
       PROGRAM-ID.    KSDUPDC1.
       AUTHOR.        SIMOTIME TECHNOLOGIES.
      *****************************************************************
      * Copyright (C) 1987-2019 SimoTime Technologies.                *
      *                                                               *
      * All rights reserved.  Unpublished, all rights reserved under  *
      * copyright law and international treaty.  Use of a copyright   *
      * notice is precautionary only and does not imply publication   *
      * or disclosure.                                                *
      *                                                               *
      * Permission to use, copy, modify and distribute this software  *
      * for any non-commercial purpose and without fee is hereby      *
      * granted, provided the SimoTime copyright notice appear on all *
      * copies of the software. The SimoTime name or Logo may not be  *
      * used in any advertising or publicity pertaining to the use    *
      * of the software without the written permission of SimoTime    *
      * Technologies.                                                 *
      *                                                               *
      * Permission to use, copy, modify and distribute this software  *
      * for any commercial purpose requires a fee to be paid to       *
      * SimoTime Technologies. Once the fee is received by SimoTime   *
      * the latest version of the software will be delivered and a    *
      * license will be granted for use within an enterprise,         *
      * provided the SimoTime copyright notice appear on all copies   *
      * of the software. The SimoTime name or Logo may not be used    *
      * in any advertising or publicity pertaining to the use of the  *
      * software without the written permission of SimoTime           *
      * Technologies.                                                 *
      *                                                               *
      * SimoTime Technologies makes no warranty or representations    *
      * about the suitability of the software for any purpose. It is  *
      * provided "AS IS" without any expressed or implied warranty,   *
      * including the implied warranties of merchantability, fitness  *
      * for a particular purpose and non-infringement. SimoTime       *
      * Technologies shall not be liable for any direct, indirect,    *
      * special or consequential damages resulting from the loss of   *
      * use, data or projects, whether in an action of contract or    *
      * tort, arising out of or in connection with the use or         *
      * performance of this software                                  *
      *                                                               *
      * SimoTime Technologies                                         *
      * 15 Carnoustie Drive                                           *
      * Novato, CA 94949-5849                                         *
      * 415.883.6565                                                  *
      *                                                               *
      * RESTRICTED RIGHTS LEGEND                                      *
      * Use, duplication, or disclosure by the Government is subject  *
      * to restrictions as set forth in subparagraph (c)(1)(ii) of    *
      * the Rights in Technical Data and Computer Software clause at  *
      * DFARS 52.227-7013 or subparagraphs (c)(1) and (2) of          *
      * Commercial  Computer Software - Restricted Rights  at 48      *
      * CFR 52.227-19, as applicable.  Contact SimoTime Technologies, *
      * 15 Carnoustie Drive, Novato, CA 94949-5849.                   *
      *                                                               *
      *****************************************************************
      *      This program is provided by SimoTime Technologies        *
      *        Our e-mail address is: helpdesk@simotime.com           *
      *     Also, visit our Web Site at http://www.simotime.com       *
      *                                                               *
      *****************************************************************
      *
      *****************************************************************
      * Source Member: KSDUPDC1.CBL
      * Copy Files:    QQSMREC1.CPY
      *****************************************************************
      *
      * KSDUPDC1 - Execute KSDUPDC1 to read a QSAM file and  update
      * a VSAM Keyed Sequential Data Set (KSDS).
      *
      * EXECUTION or CALLING PROTOCOL
      * -----------------------------
      * Use standard JCL to EXECUTE or ANIMATE.
      *
      * DESCRIPTION
      * -----------
      * This single COBOL program will read an 80-byte, EBCDIC, QSAM
      * file an populate or update an empty or existing KSDS, VSAM
      * data set.
      *
      *                      ************
      *                      * KSDUPD *
      *                      ********jcl*
      *                           *
      *                           *
      *     ************     ************     ************
      *     * QSAM0080 *-----* KSDUPDC1 *-----* VKSD0080 *
      *     *******qsam*     ********cbl*     *******vsam*
      *                           *
      *                           *
      *                      ************
      *                      *   EOJ    *
      *                      ************
      *
      *
      *****************************************************************
      *
      * MAINTENANCE
      * -----------
      * 1997/02/27 Simmons, Created program.
      * 1997/02/27 Simmons, No changes to date.
      *
      *****************************************************************
      *
       ENVIRONMENT DIVISION.
       INPUT-OUTPUT   SECTION.
       FILE-CONTROL.

      *****************************************************************
           SELECT QSAM0080-FILE
               ASSIGN       to QSAM0080
               ORGANIZATION is SEQUENTIAL
               ACCESS MODE  is SEQUENTIAL
               FILE STATUS  is QSAM0080-STATUS.
           SELECT VKSD0080-FILE
               ASSIGN       to VKSD0080
               ORGANIZATION is indexed
               ACCESS MODE  is RANDOM
               RECORD KEY   is VKSD-KEY
               FILE STATUS  is VKSD0080-STATUS.

      *****************************************************************
       DATA DIVISION.
       FILE SECTION.

      *****************************************************************
       FD   QSAM0080-FILE
            RECORD CONTAINS 80 CHARACTERS.

       COPY SQ0080B1.

       FD  VKSD0080-FILE.
       COPY VKSDREC1.

       WORKING-STORAGE SECTION.
      *****************************************************************
      *    Data-structure for Title and Copyright...
      *****************************************************************
       01  SIM-TITLE.
           05  T1 pic X(11) value '* KSDUPDC1 '.
           05  T2 pic X(34) value ' Sample, Read QSAM, Update VSAM   '.
           05  T3 pic X(10) value ' v04.04.14'.
           05  T4 pic X(24) value ' http://www.simotime.com'.
       01  SIM-COPYRIGHT.
           05  C1 pic X(11) value '* KSDUPDC1 '.
           05  C2 pic X(20) value 'Copyright 1987-2019 '.
           05  C3 pic X(28) value '   SimoTime Technologies    '.
           05  C4 pic X(20) value ' All Rights Reserved'.

       01  SIM-THANKS-01.
           05  C1 pic X(11) value '* KSDUPDC1 '.
           05  C2 pic X(32) value 'Thank you for using this program'.
           05  C3 pic X(32) value ' provided from SimoTime Technolo'.
           05  C4 pic X(04) value 'gies'.


       01  SIM-THANKS-02.
           05  C1 pic X(11) value '* KSDUPDC1 '.
           05  C2 pic X(32) value 'Please send all inquires or sugg'.
           05  C3 pic X(32) value 'estions to the helpdesk@simotime'.
           05  C4 pic X(04) value '.com'.

       01  VKSD0080-STATUS.
           05  VKSD0080-STAT1      pic X.
           05  VKSD0080-STAT2      pic X.

       01  QSAM0080-STATUS.
           05  QSAM0080-STAT1      pic X.
           05  QSAM0080-STAT2      pic X.

       01  IO-STATUS.
           05  IO-STAT1            pic X.
           05  IO-STAT2            pic X.
       01  TWO-BYTES.
           05  TWO-BYTES-LEFT      pic X.
           05  TWO-BYTES-RIGHT     pic X.
       01  TWO-BYTES-BINARY        redefines TWO-BYTES pic 9(4) comp.

       01  END-OF-FILE             pic X(3)    value 'NO '.

       01  CONSOLE-MESSAGE         pic X(48).

       01  VSAM-WRITE-ONLY         pic X       value 'N'.

       01  APPL-RESULT             pic S9(9)   comp.
           88  APPL-AOK            value 0.
           88  APPL-EOF            value 16.

      *****************************************************************
       PROCEDURE DIVISION.

           perform Z-POST-COPYRIGHT.

           perform QSAM0080-OPEN.
           perform VKSD0080-OPEN.

           perform until END-OF-FILE = 'YES'
               if  END-OF-FILE = 'NO '
                   perform QSAM0080-GET
                   if  END-OF-FILE = 'NO '
                       display QSAM-RECORD upon console
                       if  VSAM-WRITE-ONLY = 'Y'
                           move QSAM-RECORD to VKSD-RECORD
                           perform VKSD0080-WRITE
                       else
                           move QSAM-KEY to VKSD-KEY
                           perform VKSD0080-GET
                           move QSAM-RECORD to VKSD-RECORD
                           if  APPL-AOK
                               perform VKSD0080-REWRITE
                           else
                               perform VKSD0080-WRITE
                           end-if
                       end-if
                   end-if
               end-if
           end-perform.

           perform VKSD0080-CLOSE.
           perform QSAM0080-CLOSE.

           display 'KSDUPDC1 VKSD0080-HAS-BEEN-UPDATED' upon console
           display 'KSDUPDC1 NORMAL-END-OF-JOB...'      upon console

           perform Z-THANK-YOU.

           GOBACK.

      *****************************************************************
      * The following routines are in alphabetic sequence.            *
      *****************************************************************

      *****************************************************************
      * Routines to do a sequential read of the QSAM file.            *
      *****************************************************************
       QSAM0080-GET.
           read QSAM0080-FILE
           if  QSAM0080-STATUS = '00'
               subtract APPL-RESULT from APPL-RESULT
           else
               if  QSAM0080-STATUS = '10'
                   add 16 to ZERO giving APPL-RESULT
               else
                   add 12 to ZERO giving APPL-RESULT
               end-if
           end-if
           if  APPL-AOK
               CONTINUE
           else
               if  APPL-EOF
                   move 'YES' to END-OF-FILE
               else
                   move 'KSDUPDC1 QSAM0080-FAILURE-GET...'
                     to   CONSOLE-MESSAGE
                   move QSAM0080-STATUS to IO-STATUS
                   perform Z-DISPLAY-CONSOLE-MESSAGE
                   perform Z-DISPLAY-IO-STATUS
                   perform Z-ABEND-PROGRAM
               end-if
           end-if
           exit.
      *---------------------------------------------------------------*
       QSAM0080-OPEN.
           add 8 to ZERO giving APPL-RESULT.
           open input QSAM0080-FILE
           if  QSAM0080-STATUS = '00'
               subtract APPL-RESULT from APPL-RESULT
           else
               add 12 to ZERO giving APPL-RESULT
           end-if
           if  APPL-AOK
               CONTINUE
           else
               move 'KSDUPDC1 QSAM0080-FAILURE-OPEN...'
                 to CONSOLE-MESSAGE
               move QSAM0080-STATUS to IO-STATUS
               perform Z-DISPLAY-CONSOLE-MESSAGE
               perform Z-DISPLAY-IO-STATUS
               perform Z-ABEND-PROGRAM
           end-if
           exit.
      *---------------------------------------------------------------*
       QSAM0080-CLOSE.
           add 8 to ZERO giving APPL-RESULT.
           close QSAM0080-FILE
           if  QSAM0080-STATUS = '00'
               subtract APPL-RESULT from APPL-RESULT
           else
               add 12 to ZERO giving APPL-RESULT
           end-if

           if  APPL-AOK
               CONTINUE
           else
               move 'KSDUPDC1, QSAM0080, FAILURE, CLOSE...'
                 to CONSOLE-MESSAGE
               move QSAM0080-STATUS to IO-STATUS
               perform Z-DISPLAY-CONSOLE-MESSAGE
               perform Z-DISPLAY-IO-STATUS
               perform Z-ABEND-PROGRAM
           end-if
           exit.

      *****************************************************************
      * Routines to do a read by KEY of the KSDS, VSAM Data Set. If   *
      * the read is successful then the record may be updated else a  *
      * new record may be added.                                      *
      *****************************************************************
       VKSD0080-GET.
           read VKSD0080-FILE
           if  VKSD0080-STATUS = '00'
               subtract APPL-RESULT from APPL-RESULT
           else
               if  VKSD0080-STATUS = '10'
                   add 16 to ZERO giving APPL-RESULT
               else
                   add 12 to ZERO giving APPL-RESULT
               end-if
           end-if
           if  APPL-AOK
               CONTINUE
           else
               if  APPL-EOF
                   move 'YES' to END-OF-FILE
               else
                   move 'KSDUPDC1 VKSD0080-Record-NOT-Found.'
                     to   CONSOLE-MESSAGE
                   perform Z-DISPLAY-CONSOLE-MESSAGE
               end-if
           end-if
           exit.
      *---------------------------------------------------------------*
       VKSD0080-WRITE.
           write VKSD-RECORD
           if  VKSD0080-STATUS = '00'
               subtract APPL-RESULT from APPL-RESULT
           else
               if  VKSD0080-STATUS = '10'
                   add 16 to ZERO giving APPL-RESULT
               else
                   add 12 to ZERO giving APPL-RESULT
               end-if
           end-if
           if  APPL-AOK
               CONTINUE
           else
               if  APPL-EOF
                   move 'YES' to END-OF-FILE
               else
                   move 'KSDUPDC1 VKSD0080-FAILURE-WRITE...'
                     to   CONSOLE-MESSAGE
                   move VKSD0080-STATUS to IO-STATUS
                   perform Z-DISPLAY-CONSOLE-MESSAGE
                   perform Z-DISPLAY-IO-STATUS
                   perform Z-ABEND-PROGRAM
               end-if
           end-if
           exit.
      *---------------------------------------------------------------*
       VKSD0080-REWRITE.
           REWRITE VKSD-RECORD
           if  VKSD0080-STATUS = '00'
               subtract APPL-RESULT from APPL-RESULT
           else
               if  VKSD0080-STATUS = '10'
                   add 16 to ZERO giving APPL-RESULT
               else
                   add 12 to ZERO giving APPL-RESULT
               end-if
           end-if
           if  APPL-AOK
               CONTINUE
           else
               if  APPL-EOF
                   move 'YES' to END-OF-FILE
               else
                   move 'KSDUPDC1 VKSD0080-FAILURE-REWRITE...'
                     to   CONSOLE-MESSAGE
                   move VKSD0080-STATUS to IO-STATUS
                   perform Z-DISPLAY-CONSOLE-MESSAGE
                   perform Z-DISPLAY-IO-STATUS
                   perform Z-ABEND-PROGRAM
               end-if
           end-if
           exit.
      *---------------------------------------------------------------*
       VKSD0080-OPEN.
           add 8 to ZERO giving APPL-RESULT.
           open I-O VKSD0080-FILE
           if  VKSD0080-STATUS = '35'
               move 'Y' to VSAM-WRITE-ONLY
               move 'KSDUPDC1 VKSD0080-FAILURE-OPEN-IO...'
                 to CONSOLE-MESSAGE
               perform Z-DISPLAY-CONSOLE-MESSAGE
               move VKSD0080-STATUS to IO-STATUS
               perform Z-DISPLAY-IO-STATUS
               move 'KSDUPDC1 VKSD0080-ATTEMPT-OPEN-OUTPUT...'
                 to CONSOLE-MESSAGE
               perform Z-DISPLAY-CONSOLE-MESSAGE
               open output VKSD0080-FILE
           end-if
           if  VKSD0080-STATUS = '00'
               subtract APPL-RESULT from APPL-RESULT
           else
               add 12 to ZERO giving APPL-RESULT
           end-if
           if  APPL-AOK
               CONTINUE
           else
               move 'KSDUPDC1 VKSD0080-FAILURE-OPEN...'
                 to CONSOLE-MESSAGE
               move VKSD0080-STATUS to IO-STATUS
               perform Z-DISPLAY-CONSOLE-MESSAGE
               perform Z-DISPLAY-IO-STATUS
               perform Z-ABEND-PROGRAM
           end-if
           exit.
      *---------------------------------------------------------------*
       VKSD0080-CLOSE.
           add 8 to ZERO giving APPL-RESULT.
           close VKSD0080-FILE
           if  VKSD0080-STATUS = '00'
               subtract APPL-RESULT from APPL-RESULT
           else
               add 12 to ZERO giving APPL-RESULT
           end-if
           if  APPL-AOK
               CONTINUE
           else
               move 'KSDUPDC1 VKSD0080-FAILURE-CLOSE...'
                 to   CONSOLE-MESSAGE
               move VKSD0080-STATUS to IO-STATUS
               perform Z-DISPLAY-CONSOLE-MESSAGE
               perform Z-DISPLAY-IO-STATUS
               perform Z-ABEND-PROGRAM
           end-if
           exit.

      *****************************************************************
      * The following Z-Routines perform administrative tasks         *
      * for this program.                                             *
      *****************************************************************
      *
      *****************************************************************
      * ABEND the program, post a message to the console and issue    *
      * a STOP RUN.                                                   *
      *****************************************************************
       Z-ABEND-PROGRAM.
           if  CONSOLE-MESSAGE not = SPACES
               perform Z-DISPLAY-CONSOLE-MESSAGE
           end-if
           move 'KSDUPDC1 PROGRAM-IS-ABENDING...'  to CONSOLE-MESSAGE
           perform Z-DISPLAY-CONSOLE-MESSAGE
           add 12 to ZERO giving RETURN-CODE
           STOP RUN.

      *****************************************************************
      * Display the file status bytes. This routine will display as   *
      * two digits if the full two byte file status is numeric. If    *
      * second byte is non-numeric then it will be treated as a       *
      * binary number.                                                *
      *****************************************************************
       Z-DISPLAY-IO-STATUS.
           if  IO-STATUS not NUMERIC
           or  IO-STAT1 = '9'
               subtract TWO-BYTES-BINARY from TWO-BYTES-BINARY
               move IO-STAT2 to TWO-BYTES-RIGHT
               display '* KSDUPDC1 FILE-STATUS-' IO-STAT1 '/'
                       TWO-BYTES-BINARY upon console
               display '* KSDUPDC1 FILE-STATUS-' IO-STAT1 '/'
                       TWO-BYTES-BINARY
           else
               display '* KSDUPDC1 FILE-STATUS-' IO-STATUS upon console
               display '* KSDUPDC1 FILE-STATUS-' IO-STATUS
           end-if
           exit.

      *****************************************************************
       Z-DISPLAY-CONSOLE-MESSAGE.
           display CONSOLE-MESSAGE
           move all SPACES to CONSOLE-MESSAGE
           exit.

      *****************************************************************
       Z-POST-COPYRIGHT.
           display SIM-TITLE
           display SIM-COPYRIGHT
           exit.

      *****************************************************************
       Z-THANK-YOU.
           display SIM-THANKS-01
           display SIM-THANKS-02
           exit.
      *****************************************************************
      *      This example is provided by SimoTime Technologies        *
      *        Our e-mail address is: helpdesk@simotime.com           *
      *     Also, visit our Web Site at http://www.simotime.com       *
      *****************************************************************
